from dataclasses import dataclass
from typing import Union

from sigma.correlations import SigmaCorrelationRule
from sigma.pipelines.secops.mappings import get_field_mapping
from sigma.pipelines.secops.utils import determine_event_type
from sigma.processing.pipeline import ProcessingPipeline
from sigma.processing.transformations import (
    DetectionItemTransformation,
    FieldMappingTransformation,
    Transformation,
)
from sigma.rule import SigmaDetectionItem, SigmaRule

from .errors import InvalidUDMFieldError
from .validators import is_valid_udm_field


@dataclass
class EnsureValidUDMFieldsTransformation(DetectionItemTransformation):
    """
    Ensure that all fields in the detection item are valid UDM fields.
    """

    udm_schema: dict

    def apply_detection_item(self, detection_item: SigmaDetectionItem) -> None:
        if detection_item.field:
            if not is_valid_udm_field(detection_item.field, self.udm_schema):
                raise InvalidUDMFieldError(f"Field {detection_item.field} is not a valid UDM field")


@dataclass
class SetRuleEventTypeTransformation(Transformation):
    """
    Sets the event_type custom attribute on a rule, that can be used by the processing pipeline and backend during processing.
    """

    def apply(
        self,
        pipeline: ProcessingPipeline,
        rule: Union[SigmaRule, SigmaCorrelationRule],
    ) -> None:
        super().apply(pipeline, rule)
        rule.custom_attributes["event_type"] = determine_event_type(rule)


class EventTypeFieldMappingTransformation(FieldMappingTransformation):
    """
    Dynamically sets the mapping dictionary based on the Sigma rule's custom attribute 'event_type'.
    """

    def __init__(self):
        super().__init__({})  # Initialize parent class with an empty mapping for now

    def set_event_type_mapping(self, rule: SigmaRule):
        """
        Set the mapping dynamically based on the rule's custom attribute 'event_type'.
        """
        event_type = rule.custom_attributes.get("event_type", None)
        if event_type:
            self.mapping = get_field_mapping(event_type)

    def apply(
        self,
        pipeline: "sigma.processing.pipeline.ProcessingPipeline",  # noqa: F821 # type: ignore
        rule: Union[SigmaRule, SigmaCorrelationRule],  # noqa: F821 # type: ignore
    ) -> None:
        """Apply dynamic mapping before the field name transformations."""
        self.set_event_type_mapping(rule)  # Dynamically update the mapping
        super().apply(pipeline, rule)  # Call parent method to continue the transformation process



import json
from importlib import resources

from sigma.processing.conditions import DetectionItemProcessingItemAppliedCondition
from sigma.processing.pipeline import ProcessingItem, ProcessingPipeline
from sigma.processing.transformations import (
    FieldMappingTransformation,
)

from .mappings import get_common_mappings
from .transformations import (
    EnsureValidUDMFieldsTransformation,
    EventTypeFieldMappingTransformation,
    SetRuleEventTypeTransformation,
)

# LOAD UDM SCHEMA
udm_schema = json.loads(resources.read_text("sigma.pipelines.secops", "udm_field_schema.json"))

# PROCESSING ITEMS

## SET EVENT TYPE IN RULE CUSTOM ATTRIBUTE

set_event_type_proc_item = ProcessingItem(
    identifier="secops_set_event_type",
    transformation=SetRuleEventTypeTransformation(),
)


## FIELD MAPPINGS

event_type_field_mapping_proc_item = ProcessingItem(
    identifier="secops_event_type_field_mappings",
    transformation=EventTypeFieldMappingTransformation(),
)

# If field has not been mapped by event_type_field_mapping_proc_item, map using common_field_mappings
common_field_mappings_proc_item = ProcessingItem(
    identifier="secops_common_field_mappings",
    transformation=FieldMappingTransformation(get_common_mappings()),
    detection_item_conditions=[DetectionItemProcessingItemAppliedCondition("secops_event_type_field_mappings")],
    detection_item_condition_linking=any,
    detection_item_condition_negation=True,
)


# UDM VALIDATION

udm_validation_proc_item = ProcessingItem(
    identifier="secops_udm_validation",
    transformation=EnsureValidUDMFieldsTransformation(udm_schema),
)


def secops_udm_pipeline() -> ProcessingPipeline:
    return ProcessingPipeline(
        name="Google SecOps UDM Pipeline",
        priority=20,
        items=[
            set_event_type_proc_item,
            event_type_field_mapping_proc_item,
            common_field_mappings_proc_item,
            udm_validation_proc_item,
            # ProcessingItem(
            #    identifier="map_windows_event_ids",
            #    transformation=MapStringTransformation(mapping=get_windows_event_id_mapping()),
            # ),
            # Commented out for now, uncomment if needed
            # ProcessingItem(
            #     identifier="handle_unmapped_fields",
            #     transformation=DetectionItemFailureTransformation(
            #         "The field {field} is not mapped to UDM schema",
            #     ),
            #     condition=IncludeFieldCondition(fields=["*"]),
            # ),
            # ProcessingItem(
            #     identifier="ensure_critical_fields",
            #     transformation=RuleFailureTransformation(
            #         "Critical field {field} is missing",
            #     ),
            #     condition=ExcludeFieldCondition(fields=["metadata.event_type", "metadata.product_name"]),
            # ),
        ],
    )



import json


def is_valid_udm_field(field_path: str, udm_schema: dict) -> bool:
    """
    Check if a field path is valid according to the UDM schema.

    Args:
        field_path (str): The field path to validate.
        udm_schema (dict): The UDM schema to validate against.

    Returns:
        bool: True if the field path is valid, False otherwise.
    """

    parts = field_path.split(".")
    if not parts:
        return False

    # Start traversal
    current_level = None

    # Check if the first part is a noun
    if parts[0] in udm_schema["Nouns"]:
        current_level = udm_schema["TopLevelFields"]["noun"]
        parts = parts[1:]
    elif parts[0] in udm_schema["TopLevelFields"]:
        current_level = udm_schema["TopLevelFields"][parts[0]]
        parts = parts[1:]
    else:
        return False

    for part in parts:
        if isinstance(current_level, dict) and part in current_level:
            field_type = current_level[part]
            if isinstance(field_type, str):
                if field_type.startswith("@enum:"):
                    return True
                elif field_type.startswith("@"):
                    subtype_name = field_type[1:]
                    current_level = udm_schema["Subtypes"].get(subtype_name, {})
                else:
                    # Standard data type or subtype
                    if field_type in ["string", "integer", "boolean", "list"]:
                        return True
                    else:
                        current_level = udm_schema["Subtypes"].get(field_type, {})
            elif isinstance(field_type, dict):
                current_level = field_type
            else:
                return False
        else:
            return False
    return True


def is_valid_udm_field_value(field_path: str, value, udm_schema) -> bool:
    parts = field_path.split(".")
    if not parts:
        return False

    # Start traversal
    current_level = None

    if parts[0] in udm_schema["Nouns"]:
        current_level = udm_schema["TopLevelFields"]["noun"]
        parts = parts[1:]
    elif parts[0] in udm_schema["TopLevelFields"]:
        current_level = udm_schema["TopLevelFields"][parts[0]]
        parts = parts[1:]
    else:
        return False

    for part in parts[:-1]:  # Traverse until the second-to-last part
        if isinstance(current_level, dict) and part in current_level:
            field_type = current_level[part]
            if isinstance(field_type, str):
                if field_type.startswith("@"):
                    subtype_name = field_type[1:]
                    current_level = udm_schema["Subtypes"].get(subtype_name, {})
                else:
                    current_level = udm_schema["Subtypes"].get(field_type, {})
            elif isinstance(field_type, dict):
                current_level = field_type
            else:
                return False
        else:
            return False

    # Check the last part
    last_part = parts[-1]
    if isinstance(current_level, dict) and last_part in current_level:
        field_type = current_level[last_part]
        if isinstance(field_type, str):
            if field_type.startswith("@enum:"):
                enum_name = field_type[len("@enum:") :]
                enum_values = udm_schema["Enums"].get(enum_name, [])
                return value in enum_values
            elif field_type in udm_schema["Enums"]:
                enum_values = udm_schema["Enums"][field_type]
                return value in enum_values
            else:
                return check_value_type(value, field_type)
        elif isinstance(field_type, dict):
            return isinstance(value, str)  # For complex types, just check if it's a string

    return False


def check_value_type(value, expected_type):
    if expected_type == "string":
        return isinstance(value, str)
    elif expected_type in ["integer", "int64"]:
        if isinstance(value, int):
            return True
        elif isinstance(value, str):
            try:
                int(value)
                return True
            except ValueError:
                return False
        else:
            return False
    elif expected_type == "boolean":
        if isinstance(value, bool):
            return True
        elif isinstance(value, str):
            return value.lower() in ["true", "false"]
        else:
            return False
    elif expected_type == "list":
        return isinstance(value, list)
    else:
        return True  # For unknown types, assume it's valid


if __name__ == "__main__":
    # Load the UDM schema
    with open("udm_field_list_v3.json", "r", encoding="utf-8") as f:
        udm_schema = json.load(f)

    # Test cases
    test_cases = [
        ("security_result.alert_state", "ALERTING"),
        ("security_result.alert_state", "INVALID_STATE"),
        ("metadata.event_type", "PROCESS_LAUNCH"),
        ("metadata.event_type", "INVALID_EVENT_TYPE"),
        ("target.process.file.name", "example.exe"),
        ("target.process.file.full_path", "/path/to/example.exe"),
        ("network.application_protocol", "HTTP"),
        ("invalid.field.path", "some_value"),
        ("target.asset.hostname", "server01"),
        ("src.ip", "192.168.1.1"),
        ("principal.user.userid", "john.doe"),
    ]

    for field, value in test_cases:
        print(f"\nTesting field: {field} with value: {value}")
        is_valid = is_valid_udm_field(field, udm_schema)
        print(f"Field '{field}' is {'valid' if is_valid else 'invalid'}.")

        if is_valid:
            is_valid_value = is_valid_udm_field_value(field, value, udm_schema)
            print(f"Value '{value}' for field '{field}' is {'valid' if is_valid_value else 'invalid'}.")



from typing import Dict, Optional, Set

from sigma.rule import SigmaDetection, SigmaDetectionItem, SigmaRule


def _process_detection(detection_value, rule) -> Optional[str]:
    if isinstance(detection_value, SigmaDetectionItem):
        field_name = detection_value.field
        if field_name:
            return field_name
    elif isinstance(detection_value, SigmaDetection):
        for item in detection_value.detection_items:
            _process_detection(item, rule)
    else:
        return None


def get_rule_detection_fields(rule: SigmaRule) -> Set[str]:
    fields = set()
    for detection, detection_value in rule.detection.detections.items():
        field = _process_detection(detection_value, rule)
        if field:
            fields.add(field)
    return fields


def determine_event_type(rule: SigmaRule) -> str:
    category = rule.logsource.category
    service = rule.logsource.service

    process_indicators = [
        "process_creation",
        "process_access",
        "process_termination",
        "Image",
        "CommandLine",
        "ProcessId",
    ]
    network_indicators = ["network_connection", "firewall", "dns_query", "proxy", "DestinationIp", "SourceIp"]
    file_indicators = ["file_event", "file_change", "file_rename", "file_delete", "file_access", "TargetFilename"]
    auth_indicators = ["authentication", "login", "auth", "LogonType", "AuthenticationPackageName"]
    registry_indicators = ["registry_add", "registry_delete", "registry_set", "registry_event", "TargetObject"]

    fields: Set[str] = get_rule_detection_fields(rule)

    if category in process_indicators or service == "sysmon" or any(field in process_indicators for field in fields):
        return "process"
    elif (
        category in network_indicators
        or service in ["firewall", "dns"]
        or any(field in network_indicators for field in fields)
    ):
        return "network"
    elif category in file_indicators or any(field in file_indicators for field in fields):
        return "file"
    elif category in auth_indicators or any(field in auth_indicators for field in fields):
        return "authentication"
    elif category in registry_indicators or any(field in registry_indicators for field in fields):
        return "registry"
    elif service == "wmi":
        return "wmi"
    elif service in ["powershell", "powershell-classic"]:
        return "powershell"
    else:
        return "generic"


def get_windows_event_id_mapping() -> Dict[str, str]:
    return {
        "1": "PROCESS_LAUNCH",
        "3": "NETWORK_CONNECTION",
        "4": "SYSMON_SERVICE_STATE_CHANGED",
        "5": "PROCESS_TERMINATED",
        "6": "DRIVER_LOADED",
        "7": "IMAGE_LOADED",
        "8": "CREATE_REMOTE_THREAD",
        "9": "RAW_ACCESS_READ",
        "10": "PROCESS_ACCESS",
        "11": "FILE_CREATE",
        "12": "REGISTRY_EVENT",
        "13": "REGISTRY_EVENT",
        "14": "REGISTRY_EVENT",
        "15": "FILE_CREATE_STREAM_HASH",
        "16": "SYSMON_CONFIGURATION_CHANGE",
        "17": "PIPE_CREATED",
        "18": "PIPE_CONNECTED",
        "19": "WMI_EVENT",
        "20": "WMI_EVENT",
        "21": "WMI_EVENT",
        "22": "DNS_EVENT",
        "23": "FILE_DELETE",
        "24": "CLIPBOARD_CAPTURE",
        "25": "PROCESS_TAMPERING",
        "26": "FILE_DELETE_DETECTED",
        "4624": "USER_LOGIN",
        "4625": "USER_LOGIN",
        "4688": "PROCESS_LAUNCH",
        "4663": "FILE_ACCESS",
        "5156": "NETWORK_CONNECTION",
        "4656": "FILE_ACCESS",
        "4660": "FILE_DELETE",
        "4657": "REGISTRY_CHANGE",
        "4697": "SERVICE_INSTALL",
        "4720": "USER_CREATION",
        "4728": "GROUP_MEMBER_ADD",
        "4732": "GROUP_MEMBER_ADD",
        "4756": "GROUP_MEMBER_ADD",
    }


# You can remove or comment out the map_windows_event_id function if it's no longer used



class InvalidUDMFieldError(Exception):
    """
    Exception raised for invalid UDM fields.
    """

    def __init__(self, field: str):
        self.field = field
        self.message = f"Invalid UDM field: {field}"
        super().__init__(self.message)



from typing import ClassVar, Dict, Optional, Tuple, Type, Union

from sigma.conditions import (
    ConditionAND,
    ConditionFieldEqualsValueExpression,
    ConditionItem,
    ConditionNOT,
    ConditionOR,
)
from sigma.conversion.base import TextQueryBackend
from sigma.conversion.deferred import DeferredQueryExpression
from sigma.conversion.state import ConversionState
from sigma.types import (
    SigmaCompareExpression,
    SigmaString,
    SpecialChars,
)


class SecOpsBackend(TextQueryBackend):
    """Google SecOps UDM backend."""

    name: ClassVar[str] = "Google SecOps UDM backend"
    formats: Dict[str, str] = {
        "default": "Plain UDM queries",
    }
    requires_pipeline: bool = True

    precedence: ClassVar[Tuple[Type[ConditionItem], Type[ConditionItem], Type[ConditionItem]]] = (
        ConditionOR,
        ConditionAND,
        ConditionNOT,
    )

    group_expression: ClassVar[str] = "({expr})"

    token_separator: str = " "
    or_token: ClassVar[str] = "OR"
    and_token: ClassVar[str] = "AND"
    not_token: ClassVar[str] = "NOT"
    eq_token: ClassVar[str] = "="

    eq_expression: ClassVar[str] = "{field} {backend.eq_token} {value} nocase"  # Expression for field = value

    str_quote: ClassVar[str] = '"'
    escape_char: ClassVar[str] = "\\"
    wildcard_multi: ClassVar[str] = "*"
    wildcard_single: ClassVar[str] = "?"
    add_escaped: ClassVar[str] = "\\"

    re_expression: ClassVar[str] = "{field} = /{regex}/ nocase"
    re_escape_char: ClassVar[str] = "\\"
    re_escape: ClassVar[Tuple[str]] = ('"',)

    compare_op_expression: ClassVar[str] = "{field} {operator} {value}"
    compare_operators: ClassVar[Dict[SigmaCompareExpression.CompareOperators, str]] = {
        SigmaCompareExpression.CompareOperators.LT: "<",
        SigmaCompareExpression.CompareOperators.LTE: "<=",
        SigmaCompareExpression.CompareOperators.GT: ">",
        SigmaCompareExpression.CompareOperators.GTE: ">=",
    }

    field_null_expression: ClassVar[str] = '{field} = ""'
    field_exists_expression: ClassVar[str] = '{field} != ""'
    field_not_exists_expression: ClassVar[str] = '{field} = ""'

    convert_or_as_in: ClassVar[bool] = True
    convert_and_as_in: ClassVar[bool] = False
    in_expressions_allow_wildcards: ClassVar[bool] = True
    field_in_list_expression: ClassVar[str] = "{field} {op} /{list}/ nocase"
    or_in_operator: ClassVar[str] = "="
    list_separator: ClassVar[str] = "|"

    unbound_value_str_expression: ClassVar[str] = '"{value}"'
    unbound_value_num_expression: ClassVar[str] = "{value}"
    unbound_value_re_expression: ClassVar[str] = "{value}"

    # String matching operators. if none is appropriate eq_token is used.
    startswith_expression: ClassVar[Optional[str]] = "{field} = /^{value}.*/ nocase"
    endswith_expression: ClassVar[Optional[str]] = "{field} = /.*{value}$/ nocase"
    contains_expression: ClassVar[Optional[str]] = "{field} = /.*{value}.*/ nocase"
    wildcard_match_expression: ClassVar[Optional[str]] = (
        None  # Special expression if wildcards can't be matched with the eq_token operator
    )

    # cidr expressions
    cidr_expression: ClassVar[str] = (
        'net.ip_in_range_cidr({field}, "{value}")'  # CIDR expression query as format string with placeholders {field} = {value}
    )

    def __init__(self, processing_pipeline=None, **kwargs):
        super().__init__(processing_pipeline, **kwargs)

    def decide_string_quoting(self, s: SigmaString) -> bool:
        """
        Decide if string is quoted based on the pattern in the class attribute str_quote_pattern. If
        this matches (or not matches if str_quote_pattern_negation is set to True), the string is quoted.
        """
        if self.str_quote == "":  # No quoting if quoting string is empty.
            return False

        if s.contains_special() or self.wildcard_multi in s or self.wildcard_single in s:
            return False

        if self.str_quote_pattern is None:  # Always quote if pattern is not set.
            return True
        else:
            match = bool(self.str_quote_pattern.match(str(s)))
            if self.str_quote_pattern_negation:
                match = not match
            return match

    def convert_value_str(self, s: SigmaString, state: ConversionState, quote_string: bool = True) -> str:
        """Convert a SigmaString into a plain string which can be used in query.

        Override so when the wildcard is removed in startswith, endswith and contains expressions, we don't quote the string
        """
        converted = s.convert(
            self.escape_char,
            self.wildcard_multi,
            self.wildcard_single,
            self.str_quote + self.add_escaped,
            self.filter_chars,
        )
        if not quote_string:
            return converted
        if self.decide_string_quoting(s):
            return self.quote_string(converted)
        return converted

    def convert_condition_field_eq_val_str(
        self, cond: ConditionFieldEqualsValueExpression, state: ConversionState
    ) -> Union[str, DeferredQueryExpression]:
        """Conversion of field = string value expressions

        Override so when the wildcard is removed in startswith, endswith and contains expressions, we don't quote the string
        """
        try:
            quote_string = self.decide_string_quoting(cond.value)
            if (  # Check conditions for usage of 'startswith' operator
                self.startswith_expression is not None  # 'startswith' operator is defined in backend
                and cond.value.endswith(SpecialChars.WILDCARD_MULTI)  # String ends with wildcard
                and not cond.value[:-1].contains_special()  # Remainder of string doesn't contains special characters
            ):
                expr = (
                    self.startswith_expression
                )  # If all conditions are fulfilled, use 'startswith' operator instead of equal token
                value = cond.value[:-1]
            elif (  # Same as above but for 'endswith' operator: string starts with wildcard and doesn't contains further special characters
                self.endswith_expression is not None
                and cond.value.startswith(SpecialChars.WILDCARD_MULTI)
                and not cond.value[1:].contains_special()
            ):
                expr = self.endswith_expression
                value = cond.value[1:]
            elif (  # contains: string starts and ends with wildcard
                self.contains_expression is not None
                and cond.value.startswith(SpecialChars.WILDCARD_MULTI)
                and cond.value.endswith(SpecialChars.WILDCARD_MULTI)
                and not cond.value[1:-1].contains_special()
            ):
                expr = self.contains_expression
                value = cond.value[1:-1]
            elif (  # wildcard match expression: string contains wildcard
                self.wildcard_match_expression is not None and cond.value.contains_special()
            ):
                expr = self.wildcard_match_expression
                value = cond.value
            else:
                expr = self.eq_expression
                value = cond.value
            return expr.format(
                field=self.escape_and_quote_field(cond.field),
                value=self.convert_value_str(value, state, quote_string),
                backend=self,
            )
        except TypeError:  # pragma: no cover
            raise NotImplementedError(
                "Field equals string value expressions with strings are not supported by the backend."
            )

    def convert_condition_field_eq_val_num(
        self, cond: ConditionFieldEqualsValueExpression, state: ConversionState
    ) -> Union[str, DeferredQueryExpression]:
        """Conversion of field = number value expressions
        Override to add
        """
        try:
            return (
                self.escape_and_quote_field(cond.field)
                + self.token_separator
                + self.eq_token
                + self.token_separator
                + str(cond.value)
            )
        except TypeError:  # pragma: no cover
            raise NotImplementedError("Field equals numeric value expressions are not supported by the backend.")

    def convert_condition_as_in_expression(
        self, cond: Union[ConditionOR, ConditionAND], state: ConversionState
    ) -> Union[str, DeferredQueryExpression]:
        """Conversion of field in value list conditions.
        Overridden, as UDM search does not support the IN operator and we have to use the eq_token operator with a regex.
        Replace wildcards with .* and add nocase."""

        return self.field_in_list_expression.format(
            field=self.escape_and_quote_field(cond.args[0].field),
            op=self.or_in_operator if isinstance(cond, ConditionOR) else self.and_in_operator,
            list=self.list_separator.join(self.convert_value_for_in_expression(arg.value, state) for arg in cond.args),
        )

    def convert_value_for_in_expression(self, value, state):
        if isinstance(value, SigmaString):
            converted = self.convert_value_str(value, state).strip('"')
            return converted.replace("*", ".*").replace("?", ".") if value.contains_special() else converted
        return str(value).strip('"')



from functools import lru_cache
from typing import Dict


@lru_cache(maxsize=128)
def get_common_mappings() -> Dict[str, str]:
    return {
        "User": "principal.user.userid",
        "SourceHostname": "principal.hostname",
        "DestinationHostname": "target.hostname",
        "EventID": "metadata.product_event_type",
        "EventType": "metadata.event_type",
        "EventLog": "metadata.log_type",
        "Channel": "metadata.log_type",
        "Provider_Name": "metadata.product_name",
        "ServiceName": "target.process.name",
        "ServiceFileName": "target.process.file.full_path",
        "AccountName": "principal.user.user_display_name",
        "SubjectUserName": "principal.user.user_display_name",
        "SubjectDomainName": "principal.user.user_domain",
        "SubjectLogonId": "principal.user.session_id",
        "TargetUserName": "target.user.userid",
        "TargetDomainName": "target.user.user_domain",
        "TargetLogonId": "target.user.session_id",
        "IpAddress": "principal.ip",
        "IpPort": "principal.port",
        "WorkstationName": "principal.hostname",
        "Status": "metadata.status",
        "Severity": "metadata.severity",
        "Category": "metadata.event_category",
        "Hostname": "target.hostname",
        "ComputerName": "target.hostname",
    }


@lru_cache(maxsize=128)
def get_process_mappings() -> Dict[str, str]:
    return {
        "CommandLine": "target.process.command_line",
        "Image": "target.process.file.full_path",
        "ParentImage": "principal.process.file.full_path",
        "ParentCommandLine": "principal.process.command_line",
        "ProcessId": "target.process.pid",
        "ParentProcessId": "principal.process.pid",
        "IntegrityLevel": "target.process.integrity_level",
        "Hashes": "target.process.file.hash",
        "CurrentDirectory": "target.process.cwd",
        "Product": "target.process.file.pe.product",
        "Description": "target.process.file.pe.file_description",
        "Company": "target.process.file.pe.company",
        "FileVersion": "target.process.file.pe.file_version",
        "User": "target.user.userid",
        "LogonId": "target.user.session_id",
        "TerminalSessionId": "target.user.terminal_session_id",
        "CallTrace": "target.process.call_trace",
        "ParentUser": "principal.user.userid",
    }


@lru_cache(maxsize=128)
def get_network_mappings() -> Dict[str, str]:
    return {
        "SourceIp": "principal.ip",
        "DestinationIp": "target.ip",
        "SourcePort": "principal.port",
        "DestinationPort": "target.port",
        "Protocol": "network.ip_protocol",
        "Image": "network.application",
        "Initiated": "network.direction",
        "SourceIsIpv6": "principal.ip_is_ipv6",
        "DestinationIsIpv6": "target.ip_is_ipv6",
        "User": "principal.ip_user_info.userid",
        "DestinationHostname": "target.hostname",
        "SourceHostname": "principal.hostname",
        "QueryName": "network.dns.questions.name",
        "QueryResults": "network.dns.answers.data",
        "QueryStatus": "network.dns.response_code",
    }


@lru_cache(maxsize=128)
def get_file_mappings() -> Dict[str, str]:
    return {
        "TargetFilename": "target.file.full_path",
        "Image": "principal.process.file.full_path",
        "ObjectName": "target.file.full_path",
        "AccessMask": "target.file.access_mask",
        "CreationUtcTime": "target.file.creation_time",
        "PreviousCreationUtcTime": "target.file.previous_creation_time",
        "Contents": "target.file.content",
        "Hash": "target.file.hash",
        "OldName": "target.file.previous_full_path",
        "NewName": "target.file.full_path",
        "FileVersion": "target.file.pe.file_version",
        "Description": "target.file.pe.file_description",
        "Product": "target.file.pe.product",
        "Company": "target.file.pe.company",
        "OriginalFileName": "target.file.pe.original_file_name",
    }


@lru_cache(maxsize=128)
def get_authentication_mappings() -> Dict[str, str]:
    return {
        "LogonType": "principal.authentication.auth_type",
        "AuthenticationPackageName": "principal.authentication.auth_protocol",
        "TargetUserName": "target.user.userid",
        "SubjectUserName": "principal.user.user_display_name",
        "TargetOutboundUserName": "target.user.userid",
        "TargetUserSid": "target.user.user_uid",
        "TargetServerName": "target.hostname",
        "LogonProcessName": "principal.authentication.auth_service",
        "WorkstationName": "principal.hostname",
        "IpAddress": "principal.ip",
        "IpPort": "principal.port",
        "TargetInfo": "target.user.group_info",
        "LogonGuid": "principal.authentication.session_id",
    }


@lru_cache(maxsize=128)
def get_registry_mappings() -> Dict[str, str]:
    return {
        "TargetObject": "target.registry.registry_key",
        "Details": "target.registry.registry_value",
        "EventType": "metadata.event_type",
        "Image": "principal.process.file.full_path",
        "ProcessId": "principal.process.pid",
        "User": "principal.user.userid",
        "ObjectName": "target.registry.registry_key",
        "ObjectValueName": "target.registry.registry_value_name",
        "NewName": "target.registry.new_registry_key",
    }


@lru_cache(maxsize=128)
def get_dns_mappings() -> Dict[str, str]:
    return {
        "QueryName": "network.dns.questions.name",
        "QueryResults": "network.dns.answers.data",
        "QueryStatus": "network.dns.response_code",
        "record_type": "network.dns.questions.type",
        "answers": "network.dns.answers.name",
    }


def get_field_mapping(event_type: str) -> Dict[str, str]:

    event_type_mappings = {
        "process": get_process_mappings(),
        "network": get_network_mappings(),
        "file": get_file_mappings(),
        "authentication": get_authentication_mappings(),
        "registry": get_registry_mappings(),
        "dns": get_dns_mappings(),
    }

    mappings = {**event_type_mappings.get(event_type, {})}

    return mappings
